#include "para.h"
static struct stat_client *clients;
/* This must exactly match para.h */
const char *status_item_list[NUM_STAT_ITEMS] = {
	"status_bar",		"status",		"num_played",
	"mtime",		"length_min",		"length_sec",
	"file_size",		"status_flags",		"format",
	"score",		"audio_file_info1",	"audio_file_info2",
	"audio_file_info3",	"dbinfo1",		"dbinfo2",
	"dbinfo3",		"decoder_flags",	"audiod_status",
	"play_time",		"uptime",		"offset",
	"length",		"stream_start",		"current_time",
	"audiod_uptime",	"dbtool"
};
#define FOR_EACH_STAT_CLIENT(p) for ((p) = clients; (p); (p) = (p)->next)
#define FOR_EACH_STAT_ITEM(i) for (i = 0; i < NUM_STAT_ITEMS; i++)

static void dump_stat_client_list(void)
{
	struct stat_client *p;
	int i = 0;

	clog(DEBUG, "%s: dumping list\n", __func__);
	FOR_EACH_STAT_CLIENT(p)
		clog(INFO, "%s: client #%d: fd %d\n", __func__, i++, p->fd);
	clog(DEBUG, "%s: list dumped\n", __func__);
}

int stat_client_add(int fd)
{
	struct stat_client *p, *new_client = para_malloc(sizeof(struct stat_client));
	int num_clients = 1;

	clog(INFO, "%s: adding client on fd %d\n", __func__, fd);
	new_client->next = NULL;
	new_client->fd = fd;
	if (!clients) {
		/* very first client */
		clients = new_client;
		goto out;
	}
	FOR_EACH_STAT_CLIENT(p) {
		num_clients++;
		if (p->next)
			continue;
		p->next = new_client;
		goto out;
	}
out:
	clog(INFO, "%s: client #%d on fd %d added\n", __func__, num_clients, fd);
	dump_stat_client_list();
	return num_clients;
}

int dump_stat_line(char *line)
{
	struct stat_client *p, *prev = NULL; 
	char *buf = make_message("%s\n", line);
	ssize_t len = strlen(buf);
	int num_clients = 0;

	FOR_EACH_STAT_CLIENT(p) {
		int fd, ret;
//		clog(DEBUG, "%s: p=%lx\n", __func__, (long)p);
		fd = p->fd;
		ret = write(fd, buf, len);
		clog(DEBUG, "%s: dumped %s to fd %d, ret = %d\n", __func__,
			line, fd, ret);
		if (ret == len ) {
			num_clients++;
			prev = p;
			continue;
		}
		/* write error */
		close(fd);
		clog(INFO, "%s: deleting client on fd %d\n", __func__, fd);
		if (!prev) {
			clog(INFO, "%s: last client disconnected\n", __func__);
			/* last client disconnected */
			clients = NULL;
			goto out;
		}
		prev->next = p->next;
		clog(DEBUG, "%s: deleted client on fd %d, p=%lx\n", __func__,
			fd, (long) p);
		free(p);
		p = prev;
		dump_stat_client_list();
	}
out:
	free(buf);
	clog(DEBUG, "%s: %d client(s)\n", __func__, num_clients);
	return num_clients;
}

int dump_empty_status(void)
{
	int i;
	char *empty_items = para_strdup(NULL);

	FOR_EACH_STAT_ITEM(i) {
		char *tmp = make_message("%s:\n", status_item_list[i]);
		empty_items = para_strcat(empty_items, tmp);
		free(tmp);
	}
//	clog(DEBUG, "%s: empty items: %s\n", __func__, empty_items);
	i = dump_stat_line(empty_items);
	free(empty_items);
	return i;
}
